﻿using Monnit;
using Monnit.Mine;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.NetworkInformation;
using System.Net.Sockets;
using System.Windows.Forms;

namespace SampleMineApplication
{
    public partial class MainForm : Form
    {
        private MineServer _Server;
        private ExternalDataAccess _DataAccess;

        public MainForm()
        {
            InitializeComponent();
        }

        private void OnLoad(object sender, EventArgs e)
        {
            LoadIPAddresses();
            ddlProtocol.DataSource = Enum.GetValues(typeof(BaseApplication.eMineListenerProtocol));
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            int port;
            if (!int.TryParse(txtPort.Text, out port))
            {
                MessageBox.Show("Invalid Port");
            }
            else
            {
                BaseApplication.eMineListenerProtocol Protocol;
                Enum.TryParse<BaseApplication.eMineListenerProtocol>(ddlProtocol.SelectedValue.ToString(), out Protocol);

                btnStart.Enabled = false;
                ddlIpSelector.Enabled = false;
                ddlProtocol.Enabled = false;
                txtPort.Enabled = false;
                //_Server = new MineServer(Protocol, (IPAddress)ddlIpSelector.SelectedItem, port);

                /* Use this MineServer constructor to use your own implementation of ExternalDataAccess.
				 */
                _DataAccess = new ExternalDataAccess();
                _Server = new MineServer(Protocol, (IPAddress)ddlIpSelector.SelectedItem, port, _DataAccess);

                _Server.StartServer();
                PrintOutput("Server has been started");

                _Server.LogException += Handle_LogException;
                _Server.PersistGateway += Handle_PersistGateway;
                _Server.PersistSensor += Handle_PersistSensor;
                _Server.SensorMessage += Handle_SensorMessage;
                _Server.GatewayMessage += Handle_GatewayMessage;
                _Server.UnknownGateway += Handle_UnknownGateway;
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lbOutput.Items.Clear();
        }

        private void LoadIPAddresses()
        {
            ddlIpSelector.Items.Clear();
            ddlIpSelector.Items.Add(IPAddress.Any);
            ddlIpSelector.SelectedIndex = 0;

            NetworkInterface[] interfaces = NetworkInterface.GetAllNetworkInterfaces();
            IPHostEntry host;
            host = Dns.GetHostEntry(Dns.GetHostName());

            foreach (NetworkInterface adapter in interfaces)
            {
                if (adapter.OperationalStatus == OperationalStatus.Up
                    && (adapter.NetworkInterfaceType == NetworkInterfaceType.Ethernet
                        || adapter.NetworkInterfaceType == NetworkInterfaceType.Wireless80211
                        || adapter.NetworkInterfaceType == NetworkInterfaceType.GigabitEthernet))
                {
                    var ipProps = adapter.GetIPProperties();

                    foreach (var ip in ipProps.UnicastAddresses)
                    {
                        if (ip.Address.AddressFamily == AddressFamily.InterNetwork)
                        {
                            //adapter.Description.ToString() + " - " + ip.Address.ToString();
                            ddlIpSelector.Items.Add(ip.Address);
                        }
                    }
                }
            }
        }

        private void PrintOutput(string[] output)
        {
            this.BeginInvoke(new Action(() =>
            {
                for (int i = output.Length - 1; i >= 0; i--)
                {
                    lbOutput.Items.Insert(0, output[i]);
                }
                while (lbOutput.Items.Count > 500)
                    lbOutput.Items.RemoveAt(500);
            }));

        }
    
        private void PrintOutput(string output)
        {
            this.BeginInvoke(new Action(() =>
            {
                lbOutput.Items.Insert(0, output);

                while (lbOutput.Items.Count > 500)
                    lbOutput.Items.RemoveAt(500);
            }));
                
        }

        void Handle_UnknownGateway(object sender, UnknownGatewayEventArgs e)
        { 
             PrintOutput(string.Format("Unknown  GatewayID: {0}, GatewayType: {1}",e.GatewayID,e.GatewayType.ToString()));
        }

        void Handle_GatewayMessage(object sender, GatewayMessageEventArgs e)
        {
			PrintOutput(string.Format("{0} GatewayID: {1} Type:{2} MessageCount: {3}", e.GatewayMessage.ReceivedDate, e.GatewayMessage.GatewayID, (BaseApplication.eGatewayMessageType)e.GatewayMessage.MessageType, e.GatewayMessage.MessageCount));
        }

        void Handle_SensorMessage(object sender, SensorMessagesEventArgs e)
        {
			List<string> OutputList = new List<string>();
            foreach (SensorMessage message in e.SensorMessages)
            {
                OutputList.Add(string.Format("{1} SensorID: {0} State: {4} Signal: {3}% Battery: {2}v", message.SensorID, message.MessageDate, message.Voltage, message.SignalStrengthPercent(), message.State));
                foreach (Datum datum in message.Data)
                    OutputList.Add(string.Format("     {0}: {1}", datum.Description, datum.Data));
            }
            PrintOutput(OutputList.ToArray());
        }

        void Handle_PersistSensor(object sender, HandlePersistSensorEventArgs e)
        {
            PrintOutput(string.Format("SensorID {0} was updated in memory.", e.Sensor.SensorID));
        }

        void Handle_PersistGateway(object sender, HandlePersistGatewayEventArgs e)
        {
            PrintOutput(string.Format("GatewayID {0} was updated in memory.", e.Gateway.GatewayID));
            _DataAccess.UpdateGatewayEncryption(e.Gateway);
        }

        void Handle_LogException(object sender, HandleLogExceptionEventArgs e)
        {
            PrintOutput(string.Format("Exception - {0}", e.Exception.Message));
        }
    }
}